-- noinspection SqlDialectInspectionForFile
-- noinspection SqlNoDataSourceInspectionForFile
INSERT INTO crams_contact_contactrole (`name`, project_leader, read_only, support_notification) VALUES ('E_RESEARCH_BODY Admin', false, false, false);
INSERT INTO crams_contact_contactrole (`name`, project_leader, read_only, support_notification) VALUES ('E_RESEARCH_BODY_SYSTEM Admin', false, false, false);
INSERT INTO crams_contact_contactrole (`name`, project_leader, read_only, support_notification) VALUES ('E_RESEARCH_SYSTEM_DELEGATE', false, false, false);
INSERT INTO crams_contact_contactrole (`name`, project_leader, read_only, support_notification) VALUES ('E_RESEARCH_BODY_SYSTEM_Approver', false, false, false);
INSERT INTO crams_contact_contactrole (`name`, project_leader, read_only, support_notification) VALUES ('E_RESEARCH_BODY_SYSTEM_Provisioner', false, false, false);

